@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ class CControllerMediatypeList extends CController {
 			'filter_set' =>		'in 1',
 			'filter_rst' =>		'in 1',
 			'filter_name' =>	'string',
-			'filter_status' =>	'in -1,'.MEDIA_TYPE_STATUS_ACTIVE.','.MEDIA_TYPE_STATUS_DISABLED,
+			'filter_status' =>	'in -1,'.MEDIA_TYPE_STATUS_ACTIVE.','.MEDIA_TYPE_STATUS_DISABLED
 		];
 
 		$ret = $this->validateInput($fields);
@@ -46,7 +46,7 @@ class CControllerMediatypeList extends CController {
 	}
 
 	protected function checkPermissions() {
-		return ($this->getUserType() == USER_TYPE_SUPER_ADMIN);
+		return $this->checkAccess(CRoleHelper::UI_ADMINISTRATION_MEDIA_TYPES);
 	}
 
 	protected function doAction() {
@@ -70,8 +70,6 @@ class CControllerMediatypeList extends CController {
 			'status' => CProfile::get('web.media_types.filter_status', -1)
 		];
 
-		$config = select_config();
-
 		$data = [
 			'uncheck' => $this->hasInput('uncheck'),
 			'sort' => $sortField,
@@ -82,6 +80,7 @@ class CControllerMediatypeList extends CController {
 		];
 
 		// get media types
+		$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 		$data['mediatypes'] = API::Mediatype()->get([
 			'output' => ['mediatypeid', 'name', 'type', 'smtp_server', 'smtp_helo', 'smtp_email', 'exec_path',
 				'gsm_modem', 'username', 'status'
@@ -92,7 +91,7 @@ class CControllerMediatypeList extends CController {
 			'filter' => [
 				'status' => ($filter['status'] == -1) ? null : $filter['status']
 			],
-			'limit' => $config['search_limit'] + 1,
+			'limit' => $limit,
 			'editable' => true,
 			'preservekeys' => true
 		]);
