@@ -42,8 +42,8 @@ abstract class CControllerPopupItemTest extends CController {
 	 *
 	 * @var array
 	 */
-	private static $testable_item_types = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL, ITEM_TYPE_AGGREGATE,
-		ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX,
+	private static $testable_item_types = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL, ITEM_TYPE_EXTERNAL,
+		ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX,
 		ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMP, ITEM_TYPE_SCRIPT
 	];
 
@@ -100,7 +100,7 @@ abstract class CControllerPopupItemTest extends CController {
 	 * @var array
 	 */
 	protected $item_types_has_key_mandatory = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL,
-		ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_IPMI,
+		ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_IPMI,
 		ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED
 	];
 
@@ -305,9 +305,8 @@ abstract class CControllerPopupItemTest extends CController {
 
 		if ($ret && $hostid != 0) {
 			$hosts = API::Host()->get([
-				'output' => ['hostid', 'host', 'name', 'status', 'available', 'proxy_hostid', 'tls_subject',
-					'ipmi_available', 'jmx_available', 'snmp_available', 'maintenance_status', 'maintenance_type',
-					'ipmi_authtype', 'ipmi_privilege', 'ipmi_username', 'ipmi_password', 'tls_psk_identity', 'tls_psk',
+				'output' => ['hostid', 'host', 'name', 'status', 'proxy_hostid', 'tls_subject', 'maintenance_status',
+					'maintenance_type', 'ipmi_authtype', 'ipmi_privilege', 'ipmi_username', 'ipmi_password',
 					'tls_issuer', 'tls_connect'
 				],
 				'hostids' => [$hostid],
@@ -316,7 +315,7 @@ abstract class CControllerPopupItemTest extends CController {
 
 			if (!$hosts) {
 				$hosts = API::Template()->get([
-					'output' => ['templateid', 'host', 'name', 'status', 'available', 'jmx_available'],
+					'output' => ['templateid', 'host', 'name', 'status'],
 					'templateids' => [$hostid],
 					'editable' => true
 				]);
@@ -376,11 +375,13 @@ abstract class CControllerPopupItemTest extends CController {
 	/**
 	 * Function returns array of item specific properties used for item testing.
 	 *
-	 * @param array $input  Stored user input used to overwrite values retrieved from database.
+	 * @param array $input       Stored user input used to overwrite values retrieved from database.
+	 * @param bool  $for_server  Whether need to add to result an additional properties used only for connection with
+	 *                           Zabbix server.
 	 *
 	 * @return array
 	 */
-	protected function getItemTestProperties(array $input) {
+	protected function getItemTestProperties(array $input, bool $for_server = false) {
 		$data = [
 			'value_type' => $input['value_type']
 		];
@@ -459,12 +460,22 @@ abstract class CControllerPopupItemTest extends CController {
 
 				if ($this->host['status'] != HOST_STATUS_TEMPLATE) {
 					$data['host'] = [
-						'tls_psk_identity' => $this->host['tls_psk_identity'],
-						'tls_psk' => $this->host['tls_psk'],
 						'tls_issuer' => $this->host['tls_issuer'],
 						'tls_connect' => $this->host['tls_connect'],
 						'tls_subject' => $this->host['tls_subject']
 					];
+
+					if ($for_server && $this->host['tls_connect'] == HOST_ENCRYPTION_PSK) {
+						$hosts = API::Host()->get([
+							'output' => ['tls_psk_identity', 'tls_psk'],
+							'hostids' => $this->host['hostid'],
+							'editable' => true
+						]);
+						$host = reset($hosts);
+
+						$data['host']['tls_psk_identity'] = $host['tls_psk_identity'];
+						$data['host']['tls_psk'] = $host['tls_psk'];
+					}
 				}
 
 				unset($data['interface']['useip'], $data['interface']['interfaceid'], $data['interface']['ip'],
@@ -520,28 +531,18 @@ abstract class CControllerPopupItemTest extends CController {
 				$data += [
 					'key' => $input['key'],
 					'host' => [
-						'hostid' => $this->host['hostid'],
-						'available' => $this->host['available'],
-						'jmx_available' => $this->host['jmx_available']
+						'hostid' => $this->host['hostid']
 					]
 				];
 
 				if ($this->host['status'] != HOST_STATUS_TEMPLATE) {
 					$data['host'] += [
 						'maintenance_status' => $this->host['maintenance_status'],
-						'ipmi_available' => $this->host['ipmi_available'],
-						'snmp_available' => $this->host['snmp_available'],
 						'maintenance_type' => $this->host['maintenance_type']
 					];
 				}
 				break;
 
-			case ITEM_TYPE_AGGREGATE:
-				$data += [
-					'key' => $input['key']
-				];
-				break;
-
 			case ITEM_TYPE_EXTERNAL:
 				$data += [
 					'key' => $input['key']
@@ -716,8 +717,8 @@ abstract class CControllerPopupItemTest extends CController {
 				'securitylevel' => ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV,
 				'authpassphrase' => '',
 				'privpassphrase' => '',
-				'authprotocol' => ITEM_AUTHPROTOCOL_MD5,
-				'privprotocol' => ITEM_PRIVPROTOCOL_DES,
+				'authprotocol' => ITEM_SNMPV3_AUTHPROTOCOL_MD5,
+				'privprotocol' => ITEM_SNMPV3_PRIVPROTOCOL_DES,
 				'contextname' => ''
 			]
 		];
@@ -1065,12 +1066,15 @@ abstract class CControllerPopupItemTest extends CController {
 			return $formula;
 		}
 
-		$expression_data = new CTriggerExpression([
+		$expression_parser = new CExpressionParser([
+			'usermacros' => true,
+			'lldmacros' => ($this->preproc_item instanceof CItemPrototype),
 			'calculated' => true,
-			'lldmacros' => ($this->preproc_item instanceof CItemPrototype)
+			'host_macro' => true,
+			'empty_host' => true
 		]);
 
-		if (($result = $expression_data->parse($formula)) === false) {
+		if ($expression_parser->parse($formula) != CParser::PARSE_SUCCESS) {
 			// Cannot parse a calculated item formula. Return as is.
 			return $formula;
 		}
@@ -1078,29 +1082,61 @@ abstract class CControllerPopupItemTest extends CController {
 		$expression = [];
 		$pos_left = 0;
 
-		foreach ($result->getTokens() as $token) {
+		$tokens = $expression_parser->getResult()->getTokensOfTypes([
+			CExpressionParserResult::TOKEN_TYPE_USER_MACRO,
+			CExpressionParserResult::TOKEN_TYPE_LLD_MACRO,
+			CExpressionParserResult::TOKEN_TYPE_STRING,
+			CExpressionParserResult::TOKEN_TYPE_HIST_FUNCTION
+		]);
+		foreach ($tokens as $token) {
 			switch ($token['type']) {
-				case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
-				case CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO:
-				case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+				case CExpressionParserResult::TOKEN_TYPE_USER_MACRO:
+				case CExpressionParserResult::TOKEN_TYPE_LLD_MACRO:
 					if ($pos_left != $token['pos']) {
 						$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
 					}
 					$pos_left = $token['pos'] + $token['length'];
-					break;
-			}
 
-			switch ($token['type']) {
-				case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
-				case CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO:
-					$expression[] = array_key_exists($token['value'], $macros_posted)
-						? CTriggerExpression::quoteString($macros_posted[$token['value']], false)
-						: $token['value'];
+					$expression[] = array_key_exists($token['match'], $macros_posted)
+						? CExpressionParser::quoteString($macros_posted[$token['match']], false)
+						: $token['match'];
 					break;
 
-				case CTriggerExprParserResult::TOKEN_TYPE_STRING:
-					$string = strtr($token['data']['string'], $macros_posted);
-					$expression[] = CTriggerExpression::quoteString($string, false, true);
+				case CExpressionParserResult::TOKEN_TYPE_STRING:
+					if ($pos_left != $token['pos']) {
+						$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
+					}
+					$pos_left = $token['pos'] + $token['length'];
+
+					$string = strtr(CExpressionParser::unquoteString($token['match']), $macros_posted);
+					$expression[] = CExpressionParser::quoteString($string, false, true);
+					break;
+
+				case CExpressionParserResult::TOKEN_TYPE_HIST_FUNCTION:
+					foreach ($token['data']['parameters'][0]['data']['filter']['tokens'] as $filter_token) {
+						switch ($filter_token['type']) {
+							case CFilterParser::TOKEN_TYPE_USER_MACRO:
+							case CFilterParser::TOKEN_TYPE_LLD_MACRO:
+								if ($pos_left != $filter_token['pos']) {
+									$expression[] = substr($formula, $pos_left, $filter_token['pos'] - $pos_left);
+								}
+								$pos_left = $filter_token['pos'] + $filter_token['length'];
+
+								$string = strtr($filter_token['match'], $macros_posted);
+								$expression[] = CFilterParser::quoteString($string);
+								break;
+
+							case CFilterParser::TOKEN_TYPE_STRING:
+								if ($pos_left != $filter_token['pos']) {
+									$expression[] = substr($formula, $pos_left, $filter_token['pos'] - $pos_left);
+								}
+								$pos_left = $filter_token['pos'] + $filter_token['length'];
+
+								$string = strtr(CFilterParser::unquoteString($filter_token['match']), $macros_posted);
+								$expression[] = CFilterParser::quoteString($string);
+								break;
+						}
+					}
 					break;
 			}
 		}
@@ -1183,6 +1219,17 @@ abstract class CControllerPopupItemTest extends CController {
 						? $macros_posted[$macro]
 						: '';
 
+					if ($inputs['type'] == ITEM_TYPE_HTTPAGENT && $field === 'posts') {
+						if ($inputs['post_type'] == ZBX_POSTTYPE_JSON && !is_numeric($macro_value)) {
+							$macro_value = json_encode($macro_value, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
+							// Remove " wrapping.
+							$macro_value = substr($macro_value, 1, -1);
+						}
+						elseif ($inputs['post_type'] == ZBX_POSTTYPE_XML) {
+							$macro_value = htmlentities($macro_value);
+						}
+					}
+
 					$inputs[$field] = substr_replace($inputs[$field], $macro_value, $pos, strlen($macro));
 				}
 			}
