@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -55,15 +55,15 @@ if (count($data['hosts_types']) == 1) {
 		);
 
 	$table->addItem((new CDiv([
-		(new CSpan($counts[HOST_AVAILABLE_TRUE]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Available')
+		(new CSpan($counts[INTERFACE_AVAILABLE_TRUE]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Available')
 	]))->addClass(ZBX_STYLE_HOST_AVAIL_TRUE));
 
 	$table->addItem((new CDiv([
-		(new CSpan($counts[HOST_AVAILABLE_FALSE]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Not available')
+		(new CSpan($counts[INTERFACE_AVAILABLE_FALSE]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Not available')
 	]))->addClass(ZBX_STYLE_HOST_AVAIL_FALSE));
 
 	$table->addItem((new CDiv([
-		(new CSpan($counts[HOST_AVAILABLE_UNKNOWN]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Unknown')
+		(new CSpan($counts[INTERFACE_AVAILABLE_UNKNOWN]))->addClass(ZBX_STYLE_TOTALS_LIST_COUNT), _('Unknown')
 	]))->addClass(ZBX_STYLE_HOST_AVAIL_UNKNOWN));
 
 	$table->addItem((new CDiv([
@@ -80,18 +80,18 @@ else {
 		if (in_array($key, $data['hosts_types'])) {
 			$counts = $data['hosts_count'][$key];
 
-			$available_row = (new CCol($counts[HOST_AVAILABLE_TRUE]))->addClass(ZBX_STYLE_HOST_AVAIL_TRUE);
-			$not_available_row = (new CCol($counts[HOST_AVAILABLE_FALSE]))->addClass(ZBX_STYLE_HOST_AVAIL_FALSE);
-			$unknown_row = (new CCol($counts[HOST_AVAILABLE_UNKNOWN]))->addClass(ZBX_STYLE_HOST_AVAIL_UNKNOWN);
+			$available_row = (new CCol($counts[INTERFACE_AVAILABLE_TRUE]))->addClass(ZBX_STYLE_HOST_AVAIL_TRUE);
+			$not_available_row = (new CCol($counts[INTERFACE_AVAILABLE_FALSE]))->addClass(ZBX_STYLE_HOST_AVAIL_FALSE);
+			$unknown_row = (new CCol($counts[INTERFACE_AVAILABLE_UNKNOWN]))->addClass(ZBX_STYLE_HOST_AVAIL_UNKNOWN);
 			$total_row = (new CCol($data['hosts_total'][$key]))->addClass(ZBX_STYLE_HOST_AVAIL_TOTAL);
 
 			if ($data['layout'] == STYLE_HORIZONTAL) {
 				$table->addRow([$value, $available_row, $not_available_row, $unknown_row, $total_row]);
 			}
 			else {
-				$rows[HOST_AVAILABLE_TRUE][] = $available_row;
-				$rows[HOST_AVAILABLE_FALSE][] = $not_available_row;
-				$rows[HOST_AVAILABLE_UNKNOWN][] = $unknown_row;
+				$rows[INTERFACE_AVAILABLE_TRUE][] = $available_row;
+				$rows[INTERFACE_AVAILABLE_FALSE][] = $not_available_row;
+				$rows[INTERFACE_AVAILABLE_UNKNOWN][] = $unknown_row;
 				$rows['hosts_total'][] = $total_row;
 			}
 		}
@@ -99,15 +99,15 @@ else {
 
 	if ($data['layout'] == STYLE_VERTICAL) {
 		$table
-			->addRow(array_merge([_('Available')], $rows[HOST_AVAILABLE_TRUE]))
-			->addRow(array_merge([_('Not available')], $rows[HOST_AVAILABLE_FALSE]))
-			->addRow(array_merge([_('Unknown')], $rows[HOST_AVAILABLE_UNKNOWN]))
+			->addRow(array_merge([_('Available')], $rows[INTERFACE_AVAILABLE_TRUE]))
+			->addRow(array_merge([_('Not available')], $rows[INTERFACE_AVAILABLE_FALSE]))
+			->addRow(array_merge([_('Unknown')], $rows[INTERFACE_AVAILABLE_UNKNOWN]))
 			->addRow(array_merge([_('Total')], $rows['hosts_total']));
 	}
 }
 
 $output = [
-	'header' => $data['name'],
+	'name' => $data['name'],
 	'body' => $table->toString()
 ];
 
