@@ -45,7 +45,7 @@ class CMediatype extends CApiService {
 	 * @param boolean $options['count'] output only count of objects in result. ( result returned in property 'rowscount' )
 	 * @param string $options['pattern'] filter by Host name containing only give pattern
 	 * @param int $options['limit'] output will be limited to given number
-	 * @param string $options['sortfield'] output will be sorted by given property [ 'mediatypeid', 'alias' ]
+	 * @param string $options['sortfield'] output will be sorted by given property [ 'mediatypeid' ]
 	 * @param string $options['sortorder'] output will be sorted in given order [ 'ASC', 'DESC' ]
 	 * @return array
 	 */
@@ -216,9 +216,7 @@ class CMediatype extends CApiService {
 
 		$simple_interval_parser = new CSimpleIntervalParser();
 
-		$i = 0;
-		foreach ($mediatypes as $mediatype) {
-			$i++;
+		foreach ($mediatypes as $i => $mediatype) {
 			// Check if media type already exists.
 			$db_mediatype = API::getApiService()->select('media_type', [
 				'output' => ['name'],
@@ -398,10 +396,10 @@ class CMediatype extends CApiService {
 					));
 				}
 
-				if (!ctype_digit((string) $mediatype['maxattempts']) || $mediatype['maxattempts'] > 10
+				if (!ctype_digit((string) $mediatype['maxattempts']) || $mediatype['maxattempts'] > 100
 						|| $mediatype['maxattempts'] < 1) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-						'maxattempts', _s('must be between "%1$s" and "%2$s"', 1, 10)
+						'maxattempts', _s('must be between "%1$s" and "%2$s"', 1, 100)
 					));
 				}
 			}
@@ -417,15 +415,15 @@ class CMediatype extends CApiService {
 				if ($simple_interval_parser->parse($mediatype['attempt_interval']) == CParser::PARSE_SUCCESS) {
 					$attempt_interval = timeUnitToSeconds($mediatype['attempt_interval']);
 
-					if ($attempt_interval < 0 || $attempt_interval > 60) {
+					if ($attempt_interval < 0 || $attempt_interval > 3600) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-							'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 60)
+							'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 3600)
 						));
 					}
 				}
 				else {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-						'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 60)
+						'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 3600)
 					));
 				}
 			}
@@ -732,10 +730,10 @@ class CMediatype extends CApiService {
 					));
 				}
 
-				if (!ctype_digit((string) $mediatype['maxattempts']) || $mediatype['maxattempts'] > 10
+				if (!ctype_digit((string) $mediatype['maxattempts']) || $mediatype['maxattempts'] > 100
 						|| $mediatype['maxattempts'] < 1) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-						'maxattempts', _s('must be between "%1$s" and "%2$s"', 1, 10)
+						'maxattempts', _s('must be between "%1$s" and "%2$s"', 1, 100)
 					));
 				}
 			}
@@ -752,15 +750,15 @@ class CMediatype extends CApiService {
 				if ($simple_interval_parser->parse($mediatype['attempt_interval']) == CParser::PARSE_SUCCESS) {
 					$attempt_interval = timeUnitToSeconds($mediatype['attempt_interval']);
 
-					if ($attempt_interval < 0 || $attempt_interval > 60) {
+					if ($attempt_interval < 0 || $attempt_interval > 3600) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-							'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 60)
+							'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 3600)
 						));
 					}
 				}
 				else {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
-						'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 60)
+						'attempt_interval', _s('must be between "%1$s" and "%2$s"', 0, 3600)
 					));
 				}
 			}
@@ -876,7 +874,7 @@ class CMediatype extends CApiService {
 		}
 
 		if ($ins_media_type_message) {
-			DB::insertBatch('media_type_message', $ins_media_type_message);
+			DB::insert('media_type_message', $ins_media_type_message);
 		}
 
 		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MEDIA_TYPE, $mediatypes);
@@ -985,6 +983,9 @@ class CMediatype extends CApiService {
 
 				$this->validateEventMenu($mediatype, $db_mediatype);
 			}
+			else if ($db_type == MEDIA_TYPE_WEBHOOK) {
+				$webhooks_params[$mediatypeid] = [];
+			}
 
 			if (array_key_exists('message_templates', $mediatype)) {
 				$message_templates[$mediatypeid] = $mediatype['message_templates'];
@@ -1019,10 +1020,7 @@ class CMediatype extends CApiService {
 			foreach ($db_webhooks_params as $param) {
 				$mediatypeid = $param['mediatypeid'];
 
-				if (!array_key_exists($mediatypeid, $webhooks_params)) {
-					$del_media_type_param[] = $param['mediatype_paramid'];
-				}
-				elseif (!array_key_exists($param['name'], $webhooks_params[$mediatypeid])) {
+				if (!array_key_exists($param['name'], $webhooks_params[$mediatypeid])) {
 					$del_media_type_param[] = $param['mediatype_paramid'];
 				}
 				elseif ($webhooks_params[$mediatypeid][$param['name']] !== $param['value']) {
@@ -1213,28 +1211,40 @@ class CMediatype extends CApiService {
 
 		// adding message templates
 		if ($options['selectMessageTemplates'] !== null && $options['selectMessageTemplates'] != API_OUTPUT_COUNT) {
+			$message_templates = [];
 			$relation_map = $this->createRelationMap($result, 'mediatypeid', 'mediatype_messageid',
 				'media_type_message'
 			);
-			$message_templates = API::getApiService()->select('media_type_message', [
-				'output' => $options['selectMessageTemplates'],
-				'filter' => ['mediatype_messageid' => $relation_map->getRelatedIds()],
-				'preservekeys' => true
-			]);
-			$message_templates = $this->unsetExtraFields($message_templates, ['mediatype_messageid', 'mediatypeid'],
-				[]
-			);
+			$related_ids = $relation_map->getRelatedIds();
+
+			if ($related_ids) {
+				$message_templates = API::getApiService()->select('media_type_message', [
+					'output' => $options['selectMessageTemplates'],
+					'mediatype_messageids' => $related_ids,
+					'preservekeys' => true
+				]);
+				$message_templates = $this->unsetExtraFields($message_templates, ['mediatype_messageid', 'mediatypeid'],
+					[]
+				);
+			}
+
 			$result = $relation_map->mapMany($result, $message_templates, 'message_templates');
 		}
 
 		// adding users
 		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
+			$users = [];
 			$relationMap = $this->createRelationMap($result, 'mediatypeid', 'userid', 'media');
-			$users = API::User()->get([
-				'output' => $options['selectUsers'],
-				'userids' => $relationMap->getRelatedIds(),
-				'preservekeys' => true
-			]);
+			$related_ids = $relationMap->getRelatedIds();
+
+			if ($related_ids) {
+				$users = API::User()->get([
+					'output' => $options['selectUsers'],
+					'userids' => $related_ids,
+					'preservekeys' => true
+				]);
+			}
+
 			$result = $relationMap->mapMany($result, $users, 'users');
 		}
 
@@ -1285,7 +1295,7 @@ class CMediatype extends CApiService {
 		if ($type == MEDIA_TYPE_WEBHOOK) {
 			$api_input_rules['fields'] += [
 				'script' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('media_type', 'script')],
-				'timeout' =>			['type' => API_TIME_UNIT, 'length' => DB::getFieldLength('media_type', 'timeout'), 'in' => '1:60'],
+				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('media_type', 'timeout'), 'in' => '1:60'],
 				'process_tags' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_MEDIA_TYPE_TAGS_DISABLED, ZBX_MEDIA_TYPE_TAGS_ENABLED])],
 				'show_event_menu' =>	['type' => API_INT32, 'in' => implode(',', [ZBX_EVENT_MENU_HIDE, ZBX_EVENT_MENU_SHOW])],
 				// Should be checked as string not as URL because it can contain macros tags.
