@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,11 +31,6 @@ class CScreenProblem extends CScreenBase {
 	 */
 	public $data;
 
-	/**
-	 * @var array
-	 */
-	private $config;
-
 	/**
 	 * Init screen data.
 	 *
@@ -50,24 +45,6 @@ class CScreenProblem extends CScreenBase {
 			$this->data['filter']['from'] = $this->timeline['from_ts'];
 			$this->data['filter']['to'] = $this->timeline['to_ts'];
 		}
-
-		$config = select_config();
-
-		$this->config = [
-			'search_limit' => $config['search_limit'],
-			'severity_color_0' => $config['severity_color_0'],
-			'severity_color_1' => $config['severity_color_1'],
-			'severity_color_2' => $config['severity_color_2'],
-			'severity_color_3' => $config['severity_color_3'],
-			'severity_color_4' => $config['severity_color_4'],
-			'severity_color_5' => $config['severity_color_5'],
-			'severity_name_0' => $config['severity_name_0'],
-			'severity_name_1' => $config['severity_name_1'],
-			'severity_name_2' => $config['severity_name_2'],
-			'severity_name_3' => $config['severity_name_3'],
-			'severity_name_4' => $config['severity_name_4'],
-			'severity_name_5' => $config['severity_name_5']
-		];
 	}
 
 	/**
@@ -76,7 +53,6 @@ class CScreenProblem extends CScreenBase {
 	 * @param array       $options
 	 * @param array|null  $options['groupids']
 	 * @param array|null  $options['hostids']
-	 * @param array|null  $options['applicationids']
 	 * @param array|null  $options['objectids']
 	 * @param string|null $options['eventid_till']
 	 * @param int|null    $options['time_from']
@@ -108,7 +84,6 @@ class CScreenProblem extends CScreenBase {
 	 * @param array       $options
 	 * @param array|null  $options['groupids']
 	 * @param array|null  $options['hostids']
-	 * @param array|null  $options['applicationids']
 	 * @param array|null  $options['objectids']
 	 * @param string|null $options['eventid_till']
 	 * @param bool        $options['recent']
@@ -148,7 +123,6 @@ class CScreenProblem extends CScreenBase {
 	 * @param array  $filter['inventory']             (optional)
 	 * @param string $filter['inventory'][]['field']
 	 * @param string $filter['inventory'][]['value']
-	 * @param string $filter['application']           (optional)
 	 * @param string $filter['name']                  (optional)
 	 * @param int    $filter['show']                  TRIGGERS_OPTION_*
 	 * @param int    $filter['from']                  (optional) usable together with 'to' and only for
@@ -159,7 +133,6 @@ class CScreenProblem extends CScreenBase {
 	 *                                                           TRIGGERS_OPTION_(RECENT|IN)_PROBLEM
 	 * @param int    $filter['age']                   (optional) usable together with 'age_state' and only for
 	 *                                                           TRIGGERS_OPTION_(RECENT|IN)_PROBLEM
-	 * @param int    $filter['severity']              (optional)
 	 * @param array  $filter['severities']            (optional)
 	 * @param int    $filter['unacknowledged']        (optional)
 	 * @param array  $filter['tags']                  (optional)
@@ -167,24 +140,25 @@ class CScreenProblem extends CScreenBase {
 	 * @param string $filter['tags'][]['value']
 	 * @param int    $filter['show_suppressed']       (optional)
 	 * @param int    $filter['show_opdata']           (optional)
-	 * @param array  $config
-	 * @param int    $config['search_limit']
 	 * @param bool   $resolve_comments
-	 * @param bool   $resolve_urls
 	 *
 	 * @static
 	 *
 	 * @return array
 	 */
-	public static function getData(array $filter, array $config, $resolve_comments = false, $resolve_urls = false) {
-		$filter_groupids = array_key_exists('groupids', $filter) && $filter['groupids'] ? $filter['groupids'] : null;
+	public static function getData(array $filter, bool $resolve_comments = false) {
+		$filter_groupids = array_key_exists('groupids', $filter) && $filter['groupids']
+			? getSubGroups($filter['groupids'])
+			: null;
 		$filter_hostids = array_key_exists('hostids', $filter) && $filter['hostids'] ? $filter['hostids'] : null;
-		$filter_applicationids = null;
 		$filter_triggerids = array_key_exists('triggerids', $filter) && $filter['triggerids']
 			? $filter['triggerids']
 			: null;
+		$show_opdata = array_key_exists('show_opdata', $filter) && $filter['show_opdata'] != OPERATIONAL_DATA_SHOW_NONE;
 
 		if (array_key_exists('exclude_groupids', $filter) && $filter['exclude_groupids']) {
+			$exclude_groupids = getSubGroups($filter['exclude_groupids']);
+
 			if ($filter_hostids === null) {
 				// get all groups if no selected groups defined
 				if ($filter_groupids === null) {
@@ -195,7 +169,7 @@ class CScreenProblem extends CScreenBase {
 					]));
 				}
 
-				$filter_groupids = array_diff($filter_groupids, $filter['exclude_groupids']);
+				$filter_groupids = array_diff($filter_groupids, $exclude_groupids);
 
 				// get available hosts
 				$filter_hostids = array_keys(API::Host()->get([
@@ -207,7 +181,7 @@ class CScreenProblem extends CScreenBase {
 
 			$exclude_hostids = array_keys(API::Host()->get([
 				'output' => [],
-				'groupids' => $filter['exclude_groupids'],
+				'groupids' => $exclude_groupids,
 				'preservekeys' => true
 			]));
 
@@ -230,18 +204,6 @@ class CScreenProblem extends CScreenBase {
 			$filter_hostids = ($filter_hostids !== null) ? array_intersect($filter_hostids, $hostids) : $hostids;
 		}
 
-		if (array_key_exists('application', $filter) && $filter['application'] !== '') {
-			$filter_applicationids = array_keys(API::Application()->get([
-				'output' => [],
-				'groupids' => $filter_groupids,
-				'hostids' => $filter_hostids,
-				'search' => ['name' => $filter['application']],
-				'preservekeys' => true
-			]));
-			$filter_groupids = null;
-			$filter_hostids = null;
-		}
-
 		$data = [
 			'problems' => [],
 			'triggers' => []
@@ -254,11 +216,10 @@ class CScreenProblem extends CScreenBase {
 			$options = [
 				'groupids' => $filter_groupids,
 				'hostids' => $filter_hostids,
-				'applicationids' => $filter_applicationids,
 				'objectids' => $filter_triggerids,
 				'eventid_till' => $eventid_till,
 				'suppressed' => false,
-				'limit' => $config['search_limit'] + 1
+				'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 			];
 
 			if (array_key_exists('name', $filter) && $filter['name'] !== '') {
@@ -278,9 +239,6 @@ class CScreenProblem extends CScreenBase {
 					$options['time_from'] = time() - $filter['age'] * SEC_PER_DAY + 1;
 				}
 			}
-			if (array_key_exists('severity', $filter) && $filter['severity'] != TRIGGER_SEVERITY_NOT_CLASSIFIED) {
-				$options['severities'] = range($filter['severity'], TRIGGER_SEVERITY_COUNT - 1);
-			}
 			if (array_key_exists('severities', $filter)) {
 				$filter_severities = implode(',', $filter['severities']);
 				$all_severities = implode(',', range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1));
@@ -307,7 +265,7 @@ class CScreenProblem extends CScreenBase {
 				? self::getDataEvents($options)
 				: self::getDataProblems($options);
 
-			$end_of_data = (count($problems) < $config['search_limit'] + 1);
+			$end_of_data = (count($problems) < CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1);
 
 			if ($problems) {
 				$eventid_till = end($problems)['eventid'] - 1;
@@ -327,7 +285,7 @@ class CScreenProblem extends CScreenBase {
 					$seen_triggerids += $triggerids;
 
 					$options = [
-						'output' => ['priority'],
+						'output' => ['priority', 'manual_close'],
 						'selectHosts' => ['hostid'],
 						'triggerids' => array_keys($triggerids),
 						'monitored' => true,
@@ -335,18 +293,14 @@ class CScreenProblem extends CScreenBase {
 						'preservekeys' => true
 					];
 
-					$show_opdata = (array_key_exists('show_opdata', $filter)
-							&& $filter['show_opdata'] != OPERATIONAL_DATA_SHOW_NONE);
-
 					$details = (array_key_exists('details', $filter) && $filter['details'] == 1);
 
 					if ($show_opdata) {
 						$options['output'][] = 'opdata';
-						$options['selectItems'] =
-							['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'];
+						$options['selectFunctions'] = ['itemid'];
 					}
 
-					if ($resolve_comments || $resolve_urls || $show_opdata || $details) {
+					if ($resolve_comments || $show_opdata || $details) {
 						$options['output'][] = 'expression';
 					}
 
@@ -358,10 +312,6 @@ class CScreenProblem extends CScreenBase {
 						$options['output'][] = 'comments';
 					}
 
-					if ($resolve_urls) {
-						$options['output'][] = 'url';
-					}
-
 					$data['triggers'] += API::Trigger()->get($options);
 				}
 
@@ -374,9 +324,29 @@ class CScreenProblem extends CScreenBase {
 				$data['problems'] += $problems;
 			}
 		}
-		while (count($data['problems']) < $config['search_limit'] + 1 && !$end_of_data);
+		while (count($data['problems']) < CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1 && !$end_of_data);
 
-		$data['problems'] = array_slice($data['problems'], 0, $config['search_limit'] + 1, true);
+		$data['problems'] = array_slice($data['problems'], 0, CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1,
+			true
+		);
+
+		if ($show_opdata && $data['triggers']) {
+			$items = API::Item()->get([
+				'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units'],
+				'selectValueMap' => ['mappings'],
+				'triggerids' => array_keys($data['triggers']),
+				'webitems' => true,
+				'preservekeys' => true
+			]);
+
+			foreach ($data['triggers'] as &$trigger) {
+				foreach ($trigger['functions'] as $function) {
+					$trigger['items'][] = $items[$function['itemid']];
+				}
+				unset($trigger['functions']);
+			}
+			unset($trigger);
+		}
 
 		return $data;
 	}
@@ -426,8 +396,6 @@ class CScreenProblem extends CScreenBase {
 	 * @param array  $data
 	 * @param array  $data['problems']
 	 * @param array  $data['triggers']
-	 * @param array  $config
-	 * @param int    $config['search_limit']
 	 * @param string $sort
 	 * @param string $sortorder
 	 *
@@ -435,13 +403,15 @@ class CScreenProblem extends CScreenBase {
 	 *
 	 * @return array
 	 */
-	public static function sortData(array $data, array $config, $sort, $sortorder) {
+	public static function sortData(array $data, $sort, $sortorder) {
 		if (!$data['problems']) {
 			return $data;
 		}
 
 		$last_problem = end($data['problems']);
-		$data['problems'] = array_slice($data['problems'], 0, $config['search_limit'], true);
+		$data['problems'] = array_slice($data['problems'], 0, CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT),
+			true
+		);
 
 		switch ($sort) {
 			case 'host':
@@ -575,13 +545,12 @@ class CScreenProblem extends CScreenBase {
 	 * @param int   $filter['show']
 	 * @param int   $filter['show_opdata']
 	 * @param bool  $resolve_comments
-	 * @param bool  $resolve_urls
 	 *
 	 * @static
 	 *
 	 * @return array
 	 */
-	public static function makeData(array $data, array $filter, $resolve_comments = false, $resolve_urls = false) {
+	public static function makeData(array $data, array $filter, bool $resolve_comments = false) {
 		// unset unused triggers
 		$triggerids = [];
 
@@ -647,15 +616,6 @@ class CScreenProblem extends CScreenBase {
 			unset($trigger);
 		}
 
-		if ($resolve_urls) {
-			foreach ($data['problems'] as &$problem) {
-				$trigger = $data['triggers'][$problem['objectid']];
-				$trigger['eventid'] = $problem['eventid'];
-				$problem['url'] = CMacrosResolverHelper::resolveTriggerUrl($trigger, $url) ? $url : '';
-			}
-			unset($problem);
-		}
-
 		// get additional data
 		$eventids = array_keys($data['problems']);
 
@@ -707,20 +667,12 @@ class CScreenProblem extends CScreenBase {
 		$userids = $userids + $actions['userids'];
 		$data['users'] = $userids
 			? API::User()->get([
-				'output' => ['alias', 'name', 'surname'],
+				'output' => ['username', 'name', 'surname'],
 				'userids' => array_keys($userids + $actions['userids']),
 				'preservekeys' => true
 			])
 			: [];
 
-		$data['mediatypes'] = $actions['mediatypeids']
-			? API::Mediatype()->get([
-				'output' => ['name', 'maxattempts'],
-				'mediatypeids' => array_keys($actions['mediatypeids']),
-				'preservekeys' => true
-			])
-			: [];
-
 		return $data;
 	}
 
@@ -785,12 +737,27 @@ class CScreenProblem extends CScreenBase {
 		$this->dataId = 'problem';
 
 		$url = (new CUrl('zabbix.php'))->setArgument('action', 'problem.view');
+		$args = [
+			'sort' => $this->data['sort'],
+			'sortorder' => $this->data['sortorder']
+		] + $this->data['filter'];
 
-		$data = self::getData($this->data['filter'], $this->config, true);
-		$data = self::sortData($data, $this->config, $this->data['sort'], $this->data['sortorder']);
+		if ($this->data['filter']['show'] == TRIGGERS_OPTION_ALL) {
+			$args['from'] = $this->timeline['from'];
+			$args['to'] = $this->timeline['to'];
+		}
+
+		if (array_key_exists('severities', $args)) {
+			$args['severities'] = array_combine($args['severities'], $args['severities']);
+		}
+
+		array_map([$url, 'setArgument'], array_keys($args), $args);
+
+		$data = self::getData($this->data['filter'], true);
+		$data = self::sortData($data, $this->data['sort'], $this->data['sortorder']);
 
 		if ($this->data['action'] === 'problem.view') {
-			$paging = getPagingLine($data['problems'], ZBX_SORT_UP, clone $url);
+			$paging = CPagerHelper::paginate($this->page, $data['problems'], ZBX_SORT_UP, $url);
 		}
 
 		$data = self::makeData($data, $this->data['filter'], true);
@@ -817,22 +784,15 @@ class CScreenProblem extends CScreenBase {
 			$triggers_hosts = getTriggersHostsList($data['triggers']);
 		}
 
-
 		$show_opdata = $this->data['filter']['compact_view']
 			? OPERATIONAL_DATA_SHOW_NONE
 			: $this->data['filter']['show_opdata'];
 
 		if ($this->data['action'] === 'problem.view') {
-			$url_form = clone $url;
-
 			$form = (new CForm('post', 'zabbix.php'))
+				->setId('problem_form')
 				->setName('problem')
-				->cleanItems()
-				->addVar('backurl',
-					$url_form
-						->setArgument('uncheck', '1')
-						->getUrl()
-				);
+				->cleanItems();
 
 			$header_check_box = (new CColHeader(
 				(new CCheckBox('all_eventids'))
@@ -843,9 +803,7 @@ class CScreenProblem extends CScreenBase {
 				? $header_check_box->addStyle('width: 20px;')
 				: $header_check_box->addClass(ZBX_STYLE_CELL_WIDTH);
 
-			$link = $url
-				->setArgument('page', $this->data['page'])
-				->getUrl();
+			$link = $url->getUrl();
 
 			$show_timeline = ($this->data['sort'] === 'clock' && !$this->data['filter']['compact_view']
 				&& $this->data['filter']['show_timeline']);
@@ -957,10 +915,12 @@ class CScreenProblem extends CScreenBase {
 				$dependencies = getTriggerDependencies($data['triggers']);
 			}
 
-			// Create link to Problem update page.
-			$problem_update_url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'acknowledge.edit')
-				->setArgument('backurl', $url->setArgument('uncheck', '1')->getUrl());
+			$allowed = [
+				'add_comments' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS),
+				'change_severity' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY),
+				'acknowledge' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS),
+				'close' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+			];
 
 			// Add problems to table.
 			foreach ($data['problems'] as $eventid => $problem) {
@@ -991,10 +951,13 @@ class CScreenProblem extends CScreenBase {
 					$cell_r_clock = '';
 				}
 
+				$can_be_closed = ($trigger['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED && $allowed['close']);
+
 				if ($problem['r_eventid'] != 0) {
 					$value = TRIGGER_VALUE_FALSE;
 					$value_str = _('RESOLVED');
 					$value_clock = $problem['r_clock'];
+					$can_be_closed = false;
 				}
 				else {
 					$in_closing = false;
@@ -1002,6 +965,7 @@ class CScreenProblem extends CScreenBase {
 					foreach ($problem['acknowledges'] as $acknowledge) {
 						if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
 							$in_closing = true;
+							$can_be_closed = false;
 							break;
 						}
 					}
@@ -1011,12 +975,11 @@ class CScreenProblem extends CScreenBase {
 					$value_clock = $in_closing ? time() : $problem['clock'];
 				}
 
+				$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 				$cell_status = new CSpan($value_str);
 
 				// Add colors and blinking to span depending on configuration and trigger parameters.
-				addTriggerValueStyle($cell_status, $value, $value_clock,
-					$problem['acknowledged'] == EVENT_ACKNOWLEDGED
-				);
+				addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
 				// Info.
 				$info_icons = [];
@@ -1057,11 +1020,12 @@ class CScreenProblem extends CScreenBase {
 					? makeTriggerDependencies($dependencies[$trigger['triggerid']])
 					: [];
 				$description[] = (new CLinkAction($problem['name']))
+					->addClass(ZBX_STYLE_WORDWRAP)
 					->setMenuPopup(CMenuPopupHelper::getTrigger($trigger['triggerid'], $problem['eventid']));
 
 				$opdata = null;
-				if ($show_opdata != OPERATIONAL_DATA_SHOW_NONE) {
 
+				if ($show_opdata != OPERATIONAL_DATA_SHOW_NONE) {
 					if ($trigger['opdata'] === '') {
 						if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
 							$opdata = (new CCol(self::getLatestValues($trigger['items'])))->addClass('latest-values');
@@ -1098,11 +1062,11 @@ class CScreenProblem extends CScreenBase {
 					$description[] = BR();
 
 					if ($trigger['recovery_mode'] == ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION) {
-						$description[] = [_('Problem'), ': ', $trigger['expression_html'], BR()];
-						$description[] = [_('Recovery'), ': ', $trigger['recovery_expression_html']];
+						$description[] = [_('Problem'), ': ', (new CDiv($trigger['expression_html']))->addClass(ZBX_STYLE_WORDWRAP), BR()];
+						$description[] = [_('Recovery'), ': ', (new CDiv($trigger['recovery_expression_html']))->addClass(ZBX_STYLE_WORDWRAP)];
 					}
 					else {
-						$description[] = $trigger['expression_html'];
+						$description[] = (new CDiv($trigger['expression_html']))->addClass(ZBX_STYLE_WORDWRAP);
 					}
 				}
 
@@ -1129,16 +1093,20 @@ class CScreenProblem extends CScreenBase {
 				}
 
 				// Create acknowledge link.
-				$problem_update_url->setArgument('eventids', [$problem['eventid']]);
-				$acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
-				$problem_update_link = (new CLink($acknowledged ? _('Yes') : _('No'), $problem_update_url->getUrl()))
-					->addClass($acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-					->addClass(ZBX_STYLE_LINK_ALT);
+				$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity']
+						|| $allowed['acknowledge'] || $can_be_closed)
+					? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+						->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+						->addClass(ZBX_STYLE_LINK_ALT)
+						->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
+					: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+						$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+					);
 
 				// Add table row.
 				$table->addRow(array_merge($row, [
 					new CCheckBox('eventids['.$problem['eventid'].']', $problem['eventid']),
-					getSeverityCell($problem['severity'], $this->config, null, $value == TRIGGER_VALUE_FALSE),
+					getSeverityCell($problem['severity'], null, $value == TRIGGER_VALUE_FALSE),
 					$show_recovery_data ? $cell_r_clock : null,
 					$show_recovery_data ? $cell_status : null,
 					$cell_info,
@@ -1153,9 +1121,7 @@ class CScreenProblem extends CScreenBase {
 						? zbx_date2age($problem['clock'], $problem['r_clock'])
 						: zbx_date2age($problem['clock']),
 					$problem_update_link,
-					makeEventActionsIcons($problem['eventid'], $data['actions'], $data['mediatypes'], $data['users'],
-						$this->config
-					),
+					makeEventActionsIcons($problem['eventid'], $data['actions'], $data['users']),
 					$this->data['filter']['show_tags'] ? $tags[$problem['eventid']] : null
 				]), ($this->data['filter']['highlight_row'] && $value == TRIGGER_VALUE_TRUE)
 					? getSeverityFlhStyle($problem['severity'])
@@ -1164,7 +1130,12 @@ class CScreenProblem extends CScreenBase {
 			}
 
 			$footer = new CActionButtonList('action', 'eventids', [
-				'acknowledge.edit' => ['name' => _('Mass update')]
+				'popup.acknowledge.edit' => [
+					'name' => _('Mass update'),
+					'disabled' => !($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+							|| $allowed['close']
+					)
+				]
 			], 'problem');
 
 			return $this->getOutput($form->addItem([$table, $paging, $footer]), true, $this->data);
@@ -1174,23 +1145,23 @@ class CScreenProblem extends CScreenBase {
 		 * Search limit performs +1 selection to know if limit was exceeded, this will assure that csv has
 		 * "search_limit" records at most.
 		 */
-		array_splice($data['problems'], $this->config['search_limit']);
+		array_splice($data['problems'], CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT));
 
 		$csv = [];
 
-		$csv[] = [
+		$csv[] = array_filter([
 			_('Severity'),
 			_('Time'),
 			_('Recovery time'),
 			_('Status'),
 			_('Host'),
 			_('Problem'),
-			($show_opdata != OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
+			($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
 			_('Duration'),
 			_('Ack'),
 			_('Actions'),
 			_('Tags')
-		];
+		]);
 
 		$tags = makeTags($data['problems'], false);
 
@@ -1252,25 +1223,29 @@ class CScreenProblem extends CScreenBase {
 				$actions_performed[] = _('Actions').' ('.$data['actions']['actions'][$problem['eventid']]['count'].')';
 			}
 
-			$csv[] = [
-				getSeverityName($problem['severity'], $this->config),
-				zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']),
-				($problem['r_eventid'] != 0)
-					? zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock'])
-					: '',
-				$value_str,
-				implode(', ', $hosts),
-				($show_opdata == OPERATIONAL_DATA_SHOW_WITH_PROBLEM && $trigger['opdata'] !== '')
-					? $problem['name'].' ('.$opdata.')'
-					: $problem['name'],
-				($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? $opdata : null,
-				($problem['r_eventid'] != 0)
-					? zbx_date2age($problem['clock'], $problem['r_clock'])
-					: zbx_date2age($problem['clock']),
-				($problem['acknowledged'] == EVENT_ACKNOWLEDGED) ? _('Yes') : _('No'),
-				implode(', ', $actions_performed),
-				implode(', ', $tags[$problem['eventid']])
-			];
+			$row = [];
+
+			$row[] = getSeverityName($problem['severity']);
+			$row[] = zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
+			$row[] = ($problem['r_eventid'] != 0) ? zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']) : '';
+			$row[] = $value_str;
+			$row[] = implode(', ', $hosts);
+			$row[] = ($show_opdata == OPERATIONAL_DATA_SHOW_WITH_PROBLEM && $trigger['opdata'] !== '')
+				? $problem['name'].' ('.$opdata.')'
+				: $problem['name'];
+
+			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
+				$row[] = $opdata;
+			}
+
+			$row[] = ($problem['r_eventid'] != 0)
+				? zbx_date2age($problem['clock'], $problem['r_clock'])
+				: zbx_date2age($problem['clock']);
+			$row[] = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED) ? _('Yes') : _('No');
+			$row[] = implode(', ', $actions_performed);
+			$row[] = implode(', ', $tags[$problem['eventid']]);
+
+			$csv[] = $row;
 		}
 
 		return zbx_toCSV($csv);
@@ -1281,16 +1256,18 @@ class CScreenProblem extends CScreenBase {
 	 *
 	 * @static
 	 *
-	 * @param array $items  An array of trigger items.
+	 * @param array $items    An array of trigger items.
 	 * @param bool  $html
 	 *
 	 * @return array|string
 	 */
-	public static function getLatestValues(array $items, $html = true) {
+	public static function getLatestValues(array $items, bool $html = true) {
 		$latest_values = [];
 
 		$items = zbx_toHash($items, 'itemid');
-		$history_values = Manager::History()->getLastValues($items, 1, ZBX_HISTORY_PERIOD);
+		$history_values = Manager::History()->getLastValues($items, 1, timeUnitToSeconds(CSettingsHelper::get(
+			CSettingsHelper::HISTORY_PERIOD
+		)));
 
 		if ($html) {
 			$hint_table = (new CTable())->addClass('list-table');
@@ -1323,16 +1300,20 @@ class CScreenProblem extends CScreenBase {
 					new CCol($last_value['value']),
 					new CCol(
 						($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64)
-							? new CLink(_('Graph'), (new CUrl('history.php'))
-								->setArgument('action', HISTORY_GRAPH)
-								->setArgument('itemids[]', $itemid)
-								->getUrl()
-							)
-							: new CLink(_('History'), (new CUrl('history.php'))
-								->setArgument('action', HISTORY_VALUES)
-								->setArgument('itemids[]', $itemid)
-								->getUrl()
-							)
+							? (CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
+								? new CLink(_('Graph'), (new CUrl('history.php'))
+									->setArgument('action', HISTORY_GRAPH)
+									->setArgument('itemids[]', $itemid)
+									->getUrl()
+								)
+								: _('Graph'))
+							: (CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
+								? new CLink(_('History'), (new CUrl('history.php'))
+									->setArgument('action', HISTORY_VALUES)
+									->setArgument('itemids[]', $itemid)
+									->getUrl()
+								)
+								: _('History'))
 					)
 				]);
 
