@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,36 +20,31 @@
 
 
 // get language translations
-require_once dirname(__FILE__).'/include/gettextwrapper.inc.php';
-require_once dirname(__FILE__).'/include/js.inc.php';
 require_once dirname(__FILE__).'/include/locales.inc.php';
-require_once dirname(__FILE__).'/include/translateDefines.inc.php';
+require_once dirname(__FILE__).'/include/gettextwrapper.inc.php';
 
-// if we must provide language constants on language different from English
-if (isset($_GET['lang'])) {
-	if (function_exists('bindtextdomain')) {
-		// initializing gettext translations depending on language selected by user
-		$locales = zbx_locale_variants($_GET['lang']);
-		foreach ($locales as $locale) {
-			putenv('LC_ALL='.$locale);
-			putenv('LANG='.$locale);
-			putenv('LANGUAGE='.$locale);
-			if (setlocale(LC_ALL, $locale)) {
-				break;
-			}
-		}
-		bindtextdomain('frontend', 'locale');
-		bind_textdomain_codeset('frontend', 'UTF-8');
-		textdomain('frontend');
-	}
-	// numeric Locale to default
-	setlocale(LC_NUMERIC, ['C', 'POSIX', 'en', 'en_US', 'en_US.UTF-8', 'English_United States.1252', 'en_GB', 'en_GB.UTF-8']);
-}
+setupLocale(array_key_exists('lang', $_GET) ? (string) $_GET['lang'] : 'en_GB');
+
+require_once dirname(__FILE__).'/include/js.inc.php';
 
 // available scripts 'scriptFileName' => 'path relative to js/'
 $availableJScripts = [
 	'common.js' => '',
-	'dashboard.grid.js' => '',
+	'class.dashboard.js' => '',
+	'class.dashboard.page.js' => '',
+	'class.dashboard.widget.placeholder.js' => '',
+	'class.widget.js' => 'widgets/',
+	'class.widget.iterator.js' => 'widgets/',
+	'class.widget.clock.js' => 'widgets/',
+	'class.widget.graph.js' => 'widgets/',
+	'class.widget.graph-prototype.js' => 'widgets/',
+	'class.widget.map.js' => 'widgets/',
+	'class.widget.navtree.js' => 'widgets/',
+	'class.widget.paste-placeholder.js' => 'widgets/',
+	'class.widget.problems.js' => 'widgets/',
+	'class.widget.problemsbysv.js' => 'widgets/',
+	'class.widget.svggraph.js' => 'widgets/',
+	'class.widget.trigerover.js' => 'widgets/',
 	'menupopup.js' => '',
 	'gtlc.js' => '',
 	'functions.js' => '',
@@ -61,7 +56,6 @@ $availableJScripts = [
 	'multiselect.js' => '',
 	'colorpicker.js' => '',
 	'chkbxrange.js' => '',
-	'csvggraphwidget.js' => '',
 	'layout.mode.js' => '',
 	'textareaflexible.js' => '',
 	'inputsecret.js' => '',
@@ -74,7 +68,6 @@ $availableJScripts = [
 	'class.base-component.js' => '',
 	'class.bbcode.js' => '',
 	'class.calendar.js' => '',
-	'class.cclock.js' => '',
 	'class.cdate.js' => '',
 	'class.cdebug.js' => '',
 	'class.cmap.js' => '',
@@ -90,7 +83,6 @@ $availableJScripts = [
 	'class.cookie.js' => '',
 	'class.coverride.js' => '',
 	'class.crangecontrol.js' => '',
-	'class.cscreen.js' => '',
 	'class.csuggest.js' => '',
 	'class.csvggraph.js' => '',
 	'class.ctree.js' => '',
@@ -98,16 +90,16 @@ $availableJScripts = [
 	'class.overlaycollection.js' => '',
 	'class.overlay.js' => '',
 	'class.cverticalaccordion.js' => '',
-	'class.mapWidget.js' => '',
 	'class.scrollable.js' => '',
 	'class.sidebar.js' => '',
+	'class.sortable.js' => '',
 	'class.svg.canvas.js' => 'vector/',
 	'class.svg.map.js' => 'vector/',
 	'class.cviewswitcher.js' => '',
-	'class.pmaster.js' => '',
 	'class.rpc.js' => '',
 	'class.tabfilter.js' => '',
 	'class.tabfilteritem.js' => '',
+	'class.tagfilteritem.js' => '',
 	'class.template.js' => '',
 	'init.js' => '',
 	'class.tab-indicators.js' => '',
@@ -115,7 +107,11 @@ $availableJScripts = [
 	'sysmap.tpl.js' => 'templates/',
 	// page-specific scripts
 	'items.js' => 'pages/',
+	'report2.js' => 'pages/',
+	'report4.js' => 'pages/',
 	'setup.js' => 'pages/',
+	'srv_status.js' => 'pages/',
+	'monitoring.overview.js' => 'pages/',
 	'popup.condition.common.js' => 'pages/',
 	'popup.operation.common.js' => 'pages/'
 ];
@@ -127,28 +123,66 @@ $tranStrings = [
 	'class.overlay.js' => [
 		'Operation details' => _('Operation details')
 	],
-	'dashboard.grid.js' => [
-		'Edit widget' => _('Edit widget'),
-		'Add widget' => _('Add widget'),
-		'Paste widget' => _('Paste widget'),
-		'S_COPY' => _('Copy'),
-		'S_PASTE' => _('Paste'),
-		'Apply' => _('Apply'),
-		'Add' => _('Add'),
-		'Edit' => _('Edit'),
-		'Cancel' => _('Cancel'),
+	'class.dashboard.js' => [
+		'Actions' => _('Actions'),
+		'Cannot add dashboard page: maximum number of %1$d dashboard pages has been added.' => _('Cannot add dashboard page: maximum number of %1$d dashboard pages has been added.'),
+		'Cannot add widget: not enough free space on the dashboard.' => _('Cannot add widget: not enough free space on the dashboard.'),
+		'Copy' => _('Copy'),
 		'Delete' => _('Delete'),
-		'Cannot add widgets in kiosk mode' => _('Cannot add widgets in kiosk mode'),
-		'You do not have permissions to edit dashboard' => _('You do not have permissions to edit dashboard'),
+		'Failed to paste dashboard page.' => _('Failed to paste dashboard page.'),
+		'Failed to paste widget.' => _('Failed to paste widget.'),
+		'Failed to update dashboard page properties.' => _('Failed to update dashboard page properties.'),
+		'Failed to update dashboard properties.' => _('Failed to update dashboard properties.'),
+		'Failed to update widget properties.' => _('Failed to update widget properties.'),
+		'Page %1$d' => _('Page %1$d'),
+		'Paste widget' => _('Paste widget'),
+		'Properties' => _('Properties'),
+		'Start slideshow' => _('Start slideshow'),
+		'Stop slideshow' => _('Stop slideshow')
+	],
+	'class.dashboard.widget.placeholder.js' => [
 		'Add a new widget' => _('Add a new widget'),
-		'Release to create a widget.' => _('Release to create a widget.'),
 		'Click and drag to desired size.' => _('Click and drag to desired size.'),
-		'Previous page' => _('Previous page'),
-		'Next page' => _('Next page'),
+		'Release to create a widget.' => _('Release to create a widget.')
+	],
+	'class.widget.js' => [
+		'10 seconds' => _n('%1$s second', '%1$s seconds', 10),
+		'30 seconds' => _n('%1$s second', '%1$s seconds', 30),
+		'1 minute' => _n('%1$s minute', '%1$s minutes', 1),
+		'2 minutes' => _n('%1$s minute', '%1$s minutes', 2),
+		'10 minutes' => _n('%1$s minute', '%1$s minutes', 10),
+		'15 minutes' => _n('%1$s minute', '%1$s minutes', 15),
+		'Actions' => _s('Actions'),
+		'Copy' => _s('Copy'),
+		'Delete' => _s('Delete'),
+		'Edit' => _s('Edit'),
+		'No refresh' => _('No refresh'),
+		'Paste' => _s('Paste'),
+		'Refresh interval' => _s('Refresh interval')
+	],
+	'class.widget.iterator.js' => [
+		'Next page' => _s('Next page'),
+		'Previous page' => _s('Previous page'),
 		'Widget is too small for the specified number of columns and rows.' =>
-			_('Widget is too small for the specified number of columns and rows.'),
-		'Cannot add widget: not enough free space on the dashboard.' =>
-			_('Cannot add widget: not enough free space on the dashboard.')
+			_s('Widget is too small for the specified number of columns and rows.')
+	],
+	'class.widget.graph.js' => [
+		'Actions' => _s('Actions'),
+		'Download image' => _s('Download image')
+	],
+	'class.widget.navtree.js' => [
+		'Add' => _s('Add'),
+		'Add child element' => _s('Add child element'),
+		'Add multiple maps' => _s('Add multiple maps'),
+		'Apply' => _s('Apply'),
+		'Cancel' => _s('Cancel'),
+		'Edit' => _s('Edit'),
+		'Edit tree element' => _s('Edit tree element'),
+		'Remove' => _s('Remove')
+	],
+	'class.widget.svggraph.js' => [
+		'Actions' => _s('Actions'),
+		'Download image' => _s('Download image')
 	],
 	'functions.js' => [
 		'Cancel' => _('Cancel'),
@@ -251,7 +285,8 @@ $tranStrings = [
 		'S_CLOSE' => _('Close')
 	],
 	'multilineinput.js' => [
-		'S_N_SYMBOLS_REMAINING' => _('%1$s symbols remaining'),
+		'S_N_CHAR_COUNT' => _('%1$s characters'),
+		'S_N_CHAR_COUNT_REMAINING' => _('%1$s characters remaining'),
 		'S_CLICK_TO_VIEW_OR_EDIT' => _('Click to view or edit'),
 		'S_APPLY' => _('Apply'),
 		'S_CANCEL' => _('Cancel')
@@ -281,6 +316,7 @@ $tranStrings = [
 		'Create trigger' => _('Create trigger'),
 		'Create dependent item' => _('Create dependent item'),
 		'Create dependent discovery rule' => _('Create dependent discovery rule'),
+		'Dashboards' => _('Dashboards'),
 		'Delete' => _('Delete'),
 		'Delete dashboard?' => _('Delete dashboard?'),
 		'Do you wish to replace the conditional expression?' => _('Do you wish to replace the conditional expression?'),
@@ -294,30 +330,19 @@ $tranStrings = [
 		'History' => _('History'),
 		'Host' => _('Host'),
 		'Inventory' => _('Inventory'),
-		'Screens' => _('Screens'),
 		'Latest data' => _('Latest data'),
 		'Latest values' => _('Latest values'),
 		'Last hour graph' => _('Last hour graph'),
 		'Last month graph' => _('Last month graph'),
 		'Last week graph' => _('Last week graph'),
 		'Problems' => _('Problems'),
-		'Refresh interval' => _('Refresh interval'),
-		'Refresh interval multiplier' => _('Refresh interval multiplier'),
 		'Scripts' => _('Scripts'),
 		'Submap' => _('Submap'),
 		'S_TRIGGER' => _('Trigger'),
 		'URL' => _('URL'),
 		'URLs' => _('URLs'),
 		'Web' => _('Web'),
-		'No refresh' => _('No refresh'),
-		'10 seconds' => _n('%1$s second', '%1$s seconds', 10),
-		'30 seconds' => _n('%1$s second', '%1$s seconds', 30),
-		'1 minute' => _n('%1$s minute', '%1$s minutes', 1),
-		'2 minutes' => _n('%1$s minute', '%1$s minutes', 2),
-		'10 minutes' => _n('%1$s minute', '%1$s minutes', 10),
-		'15 minutes' => _n('%1$s minute', '%1$s minutes', 15),
-		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader'),
-		'Download image' => _('Download image')
+		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader')
 	],
 	'init.js' => [
 		'Debug' => _('Debug'),
@@ -328,17 +353,6 @@ $tranStrings = [
 		'No interface found' => _('No interface found'),
 		'Item type does not use interface' => _('Item type does not use interface')
 	],
-	'class.cnavtree.js' => [
-		'Edit' => _('Edit'),
-		'Remove' => _('Remove'),
-		'root' => _('root'),
-		'Edit tree element' => _('Edit tree element'),
-		'Apply' => _('Apply'),
-		'Add' => _('Add'),
-		'Cancel' => _('Cancel'),
-		'Add child element' => _('Add child element'),
-		'Add multiple maps' => _('Add multiple maps')
-	],
 	'colorpicker.js' => [
 		'S_CLOSE' => _('Close')
 	],
@@ -347,7 +361,8 @@ $tranStrings = [
 		'S_MINUTE_SHORT' => _x('m', 'minute short')
 	],
 	'common.js' => [
-		'Cancel' => _('Cancel')
+		'Cancel' => _('Cancel'),
+		'Ok' => _('Ok')
 	],
 	'component.z-select.js' => [
 		'All' => _('All')
@@ -406,6 +421,8 @@ if (empty($_GET['files'])) {
 		$files[] = 'class.notification.js';
 		$files[] = 'class.notifications.js';
 	}
+
+	$js .= 'ZBX_NOREFERER = '.ZBX_NOREFERER.";\n";
 }
 else {
 	$files = $_GET['files'];
